//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_COORDINATECOLLECTION_H_
#define _MG_COORDINATECOLLECTION_H_

#include "System/Disposable.h"

/// \defgroup MgCoordinateCollection MgCoordinateCollection
/// \ingroup Collections_Module
/// \{

class MgCoordinateIterator;

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgCoordinateCollection provides support for defining an ordered set of
/// coordinates.
class MG_GEOMETRY_API MgCoordinateCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of coordinates in the collection.
    ///
    /// \return
    /// Returns number of coordinates in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the coordinate in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to retrieve.
    ///
    /// \return
    /// Returns the coordinate in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgCoordinate* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the coordinate in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to set.
    /// \param value (MgCoordinate)
    /// Coordinate to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgCoordinate* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified coordinate to the end of the collection.
    ///
    /// \param value (MgCoordinate)
    /// Coordinate to add to the collection.
    ///
    /// \return
    /// Returns nothing
    ///
    virtual void Add(MgCoordinate* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified coordinate at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the coordinate.
    /// \param value (MgCoordinate)
    /// Coordinate to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgCoordinate* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all coordinates from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified coordinate from the collection.  Returns false if
    /// the coordinate could not be removed.
    ///
    /// \param value (MgCoordinate)
    /// Coordinate reference to remove.
    ///
    /// \return
    /// Returns true if removal was successful.
    ///
    virtual bool Remove(const MgCoordinate* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the coordinate at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the coordinate to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified coordinate, false
    /// otherwise.
    ///
    /// \param value (MgCoordinate)
    /// Coordinate reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified coordinate, false
    /// otherwise.
    ///
    virtual bool Contains(const MgCoordinate* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified coordinate in the collection or -1
    /// if the coordinate does not exist.
    ///
    /// \param value (MgCoordinate)
    /// Coordinate reference to find the index of.
    ///
    /// \return
    /// The index of the specified coordinate in the collection or -1 if the
    /// coordinate does not exist.
    ///
    virtual int IndexOf(const MgCoordinate* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgCoordinateCollection object
    ///
    MgCoordinateCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgCoordinateCollection destructor
    ///
    virtual ~MgCoordinateCollection();

public:

    /// Get the co-ordinate iterator for this collection
    MgCoordinateIterator* GetIterator();

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgCoordinateCollection* Copy();

protected:
    void Dispose()
    {
        delete this;
    }

private:

    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif
