//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGENVELOPE_H_
#define _MGENVELOPE_H_

/// \defgroup MgEnvelope MgEnvelope
/// \ingroup Geometry_Module_classes
/// \{

//////////////////////////////////////////////////////////////
/// \brief
/// MgEnvelope represents a bounding box defined in terms of a
/// lower left coordinate and an upper right coordinate.
///
class MG_GEOMETRY_API MgEnvelope : public MgDisposable
{
public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Constructs and initializes  an "null" envelope.
    ///
    MgEnvelope();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Constructs and initializes  an envelope for a region defined by one
    /// coordinate.
    ///
    /// \param coord (MgCoordinate)
    /// The MgCoordinate to use in defining the region.
    ///
    MgEnvelope(MgCoordinate* coord);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Constructs and initializes  an envelope for a region defined by two
    /// coordinates.
    ///
    /// \param coord1 (MgCoordinate)
    /// The first MgCoordinate to use in defining the region.
    /// \param coord2 (MgCoordinate)
    /// The second MgCoordinate to use in defining the region.
    ///
    MgEnvelope(MgCoordinate* coord1, MgCoordinate* coord2);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Constructs and initializes  an envelope for a region defined by two
    /// coordinates , given as four doubles
    ///
    /// \param xMin (double)
    /// Left coordinate
    /// \param yMin (double)
    /// Bottom coordinate
    /// \param xMax (double)
    /// Right coordinate
    /// \param yMax (double)
    /// Top coordinate
    ///
    MgEnvelope(double xMin, double yMin, double xMax, double yMax);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Constructs and initializes  an envelope that is a copy of another.
    ///
    /// \param envelope (MgEnvelope)
    /// The MgEnvelope to copy.
    ///
    MgEnvelope(MgEnvelope* envelope);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the lower left coordinate of the envelope.
    ///
    /// \return
    /// An MgCoordinate that defines the lower left corner of the envelope.
    ///
    MgCoordinate* GetLowerLeftCoordinate();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the upper right coordinate of the envelope.
    ///
    /// \return
    /// An MgCoordinate that defines the upper right corner of the envelope.
    ///
    MgCoordinate* GetUpperRightCoordinate();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the difference between the maximum and minimum X values.
    ///
    /// \return
    /// A double representing the width of the envelope.
    ///
    double GetWidth();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the difference between the maximum and minimum Y values.
    ///
    /// \return
    /// A double representing the height of the envelope.
    ///
    double GetHeight();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the difference between the maximum and minimum Z values.
    ///
    /// \return
    /// A double representing the depth of the envelope.
    ///
    double GetDepth();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Tests whether this Envelope is "null" or not.
    ///
    /// \return
    /// True if this envelope is "null", false otherwise.
    ///
    bool IsNull();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Makes this envelope a "null" envelope.
    ///
    void MakeNull();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Enlarges the boundary of the envelope so that it contains the given
    /// coordinate.
    ///
    /// \param coordinate (MgCoordinate)
    /// The MgCoordinate to expand this envelope to include.
    ///
    void ExpandToInclude(MgCoordinate* coordinate);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Enlarges the boundary of the envelope so that it contains the given
    /// envelope.
    ///
    /// \param envelope (MgEnvelope)
    /// The MgEnvelope to expand this envelope to include.
    ///
    void ExpandToInclude(MgEnvelope* envelope);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Tests whether the given coordinate lies in or on this envelope.
    ///
    /// \param coordinate (MgCoordinate)
    /// The MgCoordinate to test containment of.
    ///
    /// \return
    /// True if the coordinate is contained in this envelope, false otherwise.
    ///
    bool Contains(MgCoordinate* coordinate);

    //////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Tests whether the given envelope lies completely inside this
    /// Envelope (inclusive of the boundary).
    ///
    /// \param envelope (MgEnvelope)
    /// The MgEnvelope to test containment of.
    ///
    /// \return
    /// True if the envelope argument is contained in this envelope,
    /// false otherwise.
    ///
    bool Contains(MgEnvelope* envelope);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Tests if the region defined by another envelope overlaps (intersects)
    /// the region of this envelope.
    ///
    /// \param envelope (MgEnvelope)
    /// The MgEnvelope to test for intersection.
    ///
    /// \return
    /// True if the envelopes intersect, false otherwise.
    ///
    bool Intersects(MgEnvelope* envelope);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this envelope.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this envelope.
    ///
    /// \return
    /// An MgEnvelope that is a copy of this one but whose region is
    /// transformed.
    ///
    MgEnvelope* Transform(MgTransform* transform);
    
    void Grow(double offset);

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

protected:
    Ptr<MgCoordinate>   m_lowerLeft;
    Ptr<MgCoordinate>   m_upperRight;
};
/// \}

#endif //_MGENVELOPE_H_
