//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGGEOMETRY_H_
#define _MGGEOMETRY_H_

/// \defgroup MgGeometry MgGeometry
/// \ingroup Geometry_Module_classes
/// \{

////////////////////////////////////////////////////////////////
/// \brief
/// MgGeometry is an abstract base class for all of the geometry
/// types.
///
/// \remarks
/// All of the geometries are constructed by calling
/// non-static methods on an instance of MgGeometryFactory and
/// are immutable once constructed.
///
/// \note
/// The following analysis functions would throw
/// MgGeometryException if called on an instance of
/// MgMultiGeometry:
/// <ol>
///   <li>Intersection</li>
///   <li>Union</li>
///   <li>Difference</li>
///   <li>SymetricDifference</li>
///   <li>Disjoint</li>
///   <li>Touches</li>
///   <li>Intersects</li>
///   <li>Crosses</li>
///   <li>Within</li>
///   <li>Contains</li>
///   <li>Overlaps</li>
///  <li>Equals</li>
/// </ol>
///

class MG_GEOMETRY_API MgGeometry : public MgGeometricEntity
{
public:
    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the type of this geometry.
    ///
    /// \return
    /// Returns the value defined in MgGeometryType corresponding to
    /// this geometry.
    ///
    virtual int GetGeometryType() = 0;  /// __get


    ////////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a geometry that represents all points whose distance
    /// from this geometric entity is less than or equal to a given
    /// distance. By default, the distance computation is done using
    /// a linear algorithm unless a custom measure is supplied.
    ///
    /// \remarks
    /// This method returns NULL when using a negative offset with
    /// points, lines, and curves.  When using a negative offset with
    /// polygons, the method returns NULL if the offset causes the
    /// buffered polygon to shrink to zero area.
    ///
    /// \param distance (double)
    /// The distance from the geometry in units.  This value cannot be zero.
    /// \param measure (MgMeasure)
    /// The MgMeasure instance to use when computing the buffer. If null,
    /// a linear distance algorithm is used.
    ///
    /// \return
    /// An MgGeometry that represents the geometric entity's buffer, or
    /// NULL if the geometry cannot be buffered.
    ///
    virtual MgGeometry* Buffer(double distance);

    enum OffsetExtType
    {
        kFillet = 0,
        kChamfer = 1,
        kExtend = 2
    };

    ////////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a geometry that represents all points whose distance
    /// from this geometric entity is less than or equal to a given
    /// distance. By default, the distance computation is done using
    /// a linear algorithm unless a custom measure is supplied.
    /// The difference between Offset() and Buffer() is: all the points
    /// returned by Offset() is on one side (left or right) of this
    /// geometric entity.
    ///
    /// \remarks
    /// This method ONLY works for MgLineString and MgPolygon.
    /// For the MgPolygon, this function returns a geometry which has
    /// the same shape as the one returned by Buffer().
    /// For the MgLineString, this function returns a MultiLineString
    /// geometry.
    /// This method accepts both positive and negative offset value.
    /// It offset this geometric entity to the right side with a positive
    /// value, and to the left side with a negative value, by assuming
    /// the polygon is oriented in counter-clockwise order.
    /// When using a negative offset with polygon, the method returns NULL
    /// if the offset causes the offset polygon to shrink to zero area.
    ///
    /// \param distance (double)
    /// The distance from the geometry in units.  This value cannot be zero.
    ///
    /// \param extType (OffsetExtType)
    /// The offset extending curve type.
    ///
    /// \return
    /// An MgGeometry that represents the geometric entity's offset, or
    /// NULL if the geometry cannot be offset.
    ///
    virtual MgGeometry* Offset(double distance, OffsetExtType extType);

public:

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator over the coordinates included in this
    /// geometric entity.
    ///
    /// \return
    /// The coordinates of this geometric entity
    ///
    virtual MgCoordinateIterator* GetCoordinates() = 0;

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();
};
/// \}

#endif //_MGGEOMETRY_H_
