//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_GEOMETRYCOLLECTION_H_
#define _MG_GEOMETRYCOLLECTION_H_

/// \defgroup MgGeometryCollection MgGeometryCollection
/// \ingroup Collections_Module
/// \{

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgGeometryCollection provides support for defining an ordered set of
/// geometry objects.
///
/// \remarks
/// Unlike MgMultiGeometry an MgGeometryCollection is not
/// considered a Geometry and hence does not derive from MgGeometry.
class MG_GEOMETRY_API MgGeometryCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of objects in the collection.
    ///
    /// \return
    /// Returns number of objects in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the object in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the object to retrieve.
    ///
    /// \return
    /// Returns the object in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgGeometry* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the object in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the object to set.
    /// \param value (MgGeometry)
    /// Object to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgGeometry* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified object to the end of the collection.
    ///
    /// \param value (MgGeometry)
    /// Object to add to the collection.
    ///
    /// \return
    /// Returns nothing
    ///
    virtual void Add(MgGeometry* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified object at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the object.
    /// \param value (MgGeometry)
    /// Object to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgGeometry* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all objects from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified object from the collection. Returns true if removal
    /// was successful.
    ///
    /// \param value (MgGeometry)
    /// Object reference to remove.
    ///
    /// \return
    /// Returns true if object was removed.
    ///
    virtual bool Remove(const MgGeometry* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the object at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the object to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified object, false
    /// otherwise.
    ///
    /// \param value (MgGeometry)
    /// Object reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified object, false
    /// otherwise.
    ///
    virtual bool Contains(const MgGeometry* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified object in the collection or -1
    /// if the object does not exist.
    ///
    /// \param value (MgGeometry)
    /// Object reference to find the index of.
    ///
    /// \return
    /// The index of the specified object in the collection or -1 if the
    /// coordinate does not exist.
    ///
    virtual int IndexOf(const MgGeometry* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgGeometryCollection object
    ///
    MgGeometryCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgLineStringCollection destructor
    ///
    virtual ~MgGeometryCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgGeometryCollection* Copy();

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    void Dispose();

private:
    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif
