//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGLINESTRING_H_
#define _MGLINESTRING_H_

/// \defgroup MgLineString MgLineString
/// \ingroup Geometry_Module_classes
/// \{

////////////////////////////////////////////////////////////////
/// \brief
/// An MgLineString is a curve with linear interpolation between
/// points.
///
/// \remarks
/// Each consecutive pair of points defines a line
/// segment. An instance of this class is constructed by calling
/// a non-static MgGeometryFactory::CreateLineString() method
/// and, once constructed, is immutable.
///
/// <!-- Example (PHP) -->
/// \htmlinclude PHPExampleTop.html
/// The following code shows the construction of an MgLineString
/// instance containing 2 coordinates.
///
/// \code
/// $geometryFactory = new MgGeometryFactory();
/// $coordinateCollection = new MgCoordinateCollection();
/// $wktReaderWriter = new MgWktReaderWriter();
///
/// // create first coordinate
/// $coordinate = $geometryFactory->CreateCoordinateXY(0,2);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create second coordinate
/// $coordinate = $geometryFactory->CreateCoordinateXY(2,2);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create LineString
/// $lineString =
/// $geometryFactory->CreateLineString($coordinateCollection);
///
/// // print out the Agf Text string for the
/// geometry$lineStringAgfText = $wktReaderWriter->Write(lineString);
/// echo "AGF Text representation of line string: $lineStringAgfText\n";
/// \endcode
/// \htmlinclude ExampleBottom.html
///
/// <!-- Example (C#) -->
/// \htmlinclude CSharpExampleTop.html
/// \code
/// using OSGeo.MapGuide;
///
/// private MgWktReaderWriter wktReaderWriter;
/// private MgGeometryFactory geometryFactory;
/// private MgLineString ls1121;
/// // the data for 1 linestring
/// private double[,] da1121 = { { 1, 1 }, { 2, 1 } };
/// private String geometryAgfText;
///
/// public MgLineString CreateALineStringXY(double[,] lineStringData)
/// {
///     MgCoordinateCollection coords = new MgCoordinateCollection();
///     for (int i = 0; i < lineStringData.GetLength(0); i++)
///     {
///         coords.Add(geometryFactory.CreateCoordinateXY(lineStringData[i,0], lineStringData[i,1]));
///     }
///     return geometryFactory.CreateLineString(coords);
/// }
///
/// geometryFactory = new MgGeometryFactory();
/// ls1121 = CreateALineStringXY(da1121);
///
/// // print out the Agf Text string for the geometry
/// wktReaderWriter = new MgWktReaderWriter();
/// geometryAgfText = wktReaderWriter.Write(ls1121);
/// // geometryAgfText now contains:
/// // "LINESTRING XY ( 1 1, 2 1 )"
/// \endcode
/// \htmlinclude ExampleBottom.html
///
class MG_GEOMETRY_API MgLineString : public MgGeometry
{
public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator that can be used to enumerate each of the line
    /// strings coordinates.
    ///
    /// \return
    /// An MgCoordinateIterator over the coordinates.
    ///
    virtual MgCoordinateIterator* GetCoordinates();

    virtual MgCoordinate* GetStartCoordinate();  /// __get, __inherited

    virtual MgCoordinate* GetEndCoordinate();  /// __get, __inherited

    virtual int GetGeometryType();  /// __get, __inherited

    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this geometric entity.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this geometric entity.
    ///
    /// \return
    /// An MgGeometricEntity that is a copy of this one with each coordinate
    /// transformed.
    ///
    virtual MgGeometricEntity* Transform(MgTransform* transform);

public:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgLineString object
    ///
    MgLineString(MgCoordinateCollection* coordinates);

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgLineString object for deserialization
    ///
    MgLineString();

protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    /// \return
    /// MgEnvelope.
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgCoordinateCollection>   m_coordinates;
};
/// \}

#endif //_MGLINESTRING_H_
