//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_LINESTRINGCOLLECTION_H_
#define _MG_LINESTRINGCOLLECTION_H_

/// \defgroup MgLineStringCollection MgLineStringCollection
/// \ingroup Collections_Module
/// \{

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgLineStringCollection provides support for defining an ordered set of
/// line strings.
class MG_GEOMETRY_API MgLineStringCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of line strings in the collection.
    ///
    /// \return
    /// Returns number of line strings in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the line string in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the line string to retrieve.
    ///
    /// \return
    /// Returns the line string in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgLineString* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the line string in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to set.
    /// \param value (MgLineString)
    /// line string to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgLineString* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified coordinate to the end of the collection.
    ///
    /// \param value (MgLineString)
    /// line string to add to the collection.
    ///
    /// \return
    /// Returns nothing.
    ///
    virtual void Add(MgLineString* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified line string at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the coordinate.
    /// \param value (MgLineString)
    /// line string to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgLineString* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all line strings from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified line string from the collection.
    ///
    /// \param value (MgLineString)
    /// line string reference to remove.
    ///
    /// \return
    /// Returns true if removal was successful.
    ///
    virtual bool Remove(const MgLineString* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the line string at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the line string to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified line string, false
    /// otherwise.
    ///
    /// \param value (MgLineString)
    /// line string reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified line string, false
    /// otherwise.
    ///
    virtual bool Contains(const MgLineString* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified line string in the collection or -1
    /// if the line string does not exist.
    ///
    /// \param value (MgLineString)
    /// line string reference to find the index of.
    ///
    /// \return
    /// The index of the specified line string in the collection or -1 if the
    /// line string does not exist.
    ///
    virtual int IndexOf(const MgLineString* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgLineStringCollection object
    ///
    MgLineStringCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgLineStringCollection destructor
    ///
    virtual ~MgLineStringCollection();


public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgLineStringCollection* Copy();


protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    void Dispose();

private:
    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif // _MG_LINESTRINGCOLLECTION_H_
