//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGLINEARRING_H_
#define _MGLINEARRING_H_

/// \defgroup MgLinearRing MgLinearRing
/// \ingroup Geometry_Module_classes
/// \{

///////////////////////////////////////////////////////////////////////////////////////////////
/// \brief
/// An MgLinearRing is a geometry component.
///
/// \remarks
/// It is used in the
/// construction of MgPolygon geometries. It is a closed shape
/// where the curves connecting the coordinates are interpolated
/// linearly. An instance of this class is constructed by calling
/// a non-static MgGeometryFaactory::CreateLinearRing() method
/// and, once constructed, is immutable. See the \link MgPolygon MgPolygon Class \endlink
/// for example code showing the construction of an MgLinearRing
/// instance.
///
class MG_GEOMETRY_API MgLinearRing : public MgRing
{

public:

    //////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator that can be used to enumerate each of
    /// the linear rings coordinates.
    ///
    /// \return
    /// An MgCoordinateIterator over the rings coordinates.
    ///
    virtual MgCoordinateIterator* GetCoordinates();


    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this geometric entity.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this geometric entity.
    ///
    /// \return
    /// An MgGeometricEntity that is a copy of this one with each coordinate
    /// transformed.
    ///
    virtual MgGeometricEntity* Transform(MgTransform* transform);

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgLinearRing object
    ///
    MgLinearRing(MgCoordinateCollection* coordinates);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgLinearRing object for deserialization
    ///
    MgLinearRing();


    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the coordinates for this ring
    /// rings coordinates.
    ///
    virtual MgCoordinateCollection* GetCoordinateCollection();


protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgCoordinateCollection> m_coordinates;
};
///\}

#endif //_MGLINEARRING_H_
