//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_LINEARRINGCOLLECTION_H_
#define _MG_LINEARRINGCOLLECTION_H_

/// \defgroup MgLinearRingCollection MgLinearRingCollection
/// \ingroup Collections_Module
/// \{

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgLinearRingCollection provides support for defining an ordered set of
/// linear rings.
class MG_GEOMETRY_API MgLinearRingCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of linear rings in the collection.
    ///
    /// \return
    /// Returns number of linear rings in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the linear ring in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the linear ring to retrieve.
    ///
    /// \return
    /// Returns the linear ring in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgLinearRing* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the linear ring in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to set.
    /// \param value (MgLinearRing)
    /// linear ring to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgLinearRing* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified coordinate to the end of the collection.
    ///
    /// \param value (MgLinearRing)
    /// linear ring to add to the collection.
    ///
    /// \return
    /// Returns nothing
    ///
    virtual void Add(MgLinearRing* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified linear ring at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the coordinate.
    /// \param value (MgLinearRing)
    /// linear ring to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgLinearRing* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all linear rings from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified linear ring from the collection. Returns true
    /// if removal was successful.
    ///
    /// \param value (MgLinearRing)
    /// linear ring reference to remove.
    ///
    /// \return
    /// Returns true if successful.
    ///
    virtual bool Remove(const MgLinearRing* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the linear ring at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the linear ring to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified linear ring, false
    /// otherwise.
    ///
    /// \param value (MgLinearRing)
    /// linear ring reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified linear ring, false
    /// otherwise.
    ///
    virtual bool Contains(const MgLinearRing* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified linear ring in the collection or -1
    /// if the linear ring does not exist.
    ///
    /// \param value (MgLinearRing)
    /// linear ring reference to find the index of.
    ///
    /// \return
    /// The index of the specified linear ring in the collection or -1 if the
    /// linear ring does not exist.
    ///
    virtual int IndexOf(const MgLinearRing* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgLinearRingCollection object
    ///
    MgLinearRingCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgLinearRingCollection destructor
    ///
    virtual ~MgLinearRingCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgLinearRingCollection* Copy();

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    void Dispose();

private:

    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif // _MG_LINEARRINGCOLLECTION_H_
