//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#include "stdafx.h"

//////////////////////////////////////////////////////////////////
// Construct a MgMultiGeometry object
//
MgMultiGeometry::MgMultiGeometry(MgGeometryCollection* geometries)
{
    m_geometries = geometries->Copy();
}

//////////////////////////////////////////////////////////////////
// Construct an empty MgMultiGeometry object for deserialization
//
MgMultiGeometry::MgMultiGeometry()
{
}

///////////////////////////////////////////////////////////////////////////
// Gets the number of geometries in this aggregate.
//
int MgMultiGeometry::GetCount()
{
    return m_geometries->GetCount();
}

///////////////////////////////////////////////////////////////////////////
// Gets the geometry at the specified index. The indexing starts at
// zero.
//
MgGeometry* MgMultiGeometry::GetGeometry(int index)
{
    return m_geometries->GetItem(index);
}

///////////////////////////////////////////////////////////////////////////
// Gets the type of this geometry.
//
int MgMultiGeometry::GetGeometryType()
{
    return MgGeometryType::MultiGeometry;
}


///////////////////////////////////////////////////////////////////////////
// Tests whether this entity is empty or not.
//
bool MgMultiGeometry::IsEmpty()
{
    return false;
}

///////////////////////////////////////////////////////////////////////////
// Tests whether the entities boundary encloses an area.
//
bool MgMultiGeometry::IsClosed()
{
    return false;
}

///////////////////////////////////////////////////////////////////////////
// Returns a copy of this geometric entity.
//
MgGeometricEntity* MgMultiGeometry::Copy()
{
    Ptr<MgGeometryCollection> geometries = m_geometries->Copy();
    return new MgMultiGeometry(geometries);
}

///////////////////////////////////////////////////////////////////////////
// Returns a transformed copy of this geometric entity.
//
MgGeometricEntity* MgMultiGeometry::Transform(MgTransform* transform)
{
    Ptr<MgGeometryCollection> geometries = new MgGeometryCollection();

    int count = m_geometries->GetCount();
    for (int i = 0; i < count; i++)
    {
        Ptr<MgGeometry> geometry = m_geometries->GetItem(i);
        Ptr<MgGeometry> newGeometry = (MgGeometry*)geometry->Transform(transform);
        geometries->Add(newGeometry);
    }

    return new MgMultiGeometry(geometries);
}


//////////////////////////////////////////////////////////////////
// Calculate the envelope for this geometry
//
MgEnvelope* MgMultiGeometry::ComputeEnvelope()
{
    if(m_envelope == NULL)
    {
        m_envelope = MgGeometryUtil::ComputeAggregateEnvelope(this);
    }
    return new MgEnvelope(m_envelope);
}

//////////////////////////////////////////////
// Dispose this object.
//
void MgMultiGeometry::Dispose()
{
    delete this;
}

///////////////////////////////////////////////////////////////////////////
// Gets the geometry at the specified position
//
MgGeometry* MgMultiGeometry::GetGeometryAt(int index)
{
    return GetGeometry(index);
}


MgCoordinateIterator* MgMultiGeometry::GetCoordinates()
{
    Ptr<MgCoordinateCollection> coords = new MgCoordinateCollection();
    int count = m_geometries->GetCount();

    for (int i = 0; i < count; i++)
    {
        Ptr<MgGeometry> geom = m_geometries->GetItem(i);
        Ptr<MgCoordinateIterator> geomIterator = geom->GetCoordinates();

        while (geomIterator->MoveNext())
        {
            coords->Add(Ptr<MgCoordinate>(geomIterator->GetCurrent()));
        }
    }

    return new MgCoordinateIterator(coords);
}
