//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#include "stdafx.h"

//////////////////////////////////////////////////////////////////
// Construct a MgMultiLineString object
//
MgMultiLineString::MgMultiLineString(MgLineStringCollection* lineStrings)
{
    m_lineStrings = lineStrings->Copy();
}

//////////////////////////////////////////////////////////////////
// Construct an empty MgMultiLineString object for deserialization
//
MgMultiLineString::MgMultiLineString()
{
}

///////////////////////////////////////////////////////////////////////////
// Gets the number of geometries in this aggregate.
//
int MgMultiLineString::GetCount()
{
    return m_lineStrings->GetCount();
}

///////////////////////////////////////////////////////////////////////////
// Gets the line string at the specified index. The indexing starts at
// zero.
//
MgLineString* MgMultiLineString::GetLineString(int index)
{
    return m_lineStrings->GetItem(index);
}

///////////////////////////////////////////////////////////////////////////
// Gets the type of this geometry.
//
int MgMultiLineString::GetGeometryType()
{
    return MgGeometryType::MultiLineString;
}

///////////////////////////////////////////////////////////////////////////
// Tests whether this entity is empty or not.
//
bool MgMultiLineString::IsEmpty()
{
    return false;
}

///////////////////////////////////////////////////////////////////////////
// Tests whether the entities boundary encloses an area.
//
bool MgMultiLineString::IsClosed()
{
    return false;
}

///////////////////////////////////////////////////////////////////////////
// Returns a copy of this geometric entity.
//
MgGeometricEntity* MgMultiLineString::Copy()
{
    Ptr<MgLineStringCollection> lineStrings = m_lineStrings->Copy();
    return new MgMultiLineString(lineStrings);
}

///////////////////////////////////////////////////////////////////////////
// Returns a transformed copy of this geometric entity.
//
MgGeometricEntity* MgMultiLineString::Transform(MgTransform* transform)
{
    Ptr<MgLineStringCollection> lineStrings = new MgLineStringCollection();

    int count = m_lineStrings->GetCount();
    for (int i = 0; i < count; i++)
    {
        Ptr<MgLineString> lineString = m_lineStrings->GetItem(i);
        Ptr<MgLineString> newLineString = (MgLineString*)lineString->Transform(transform);
        lineStrings->Add(newLineString);
    }

    return new MgMultiLineString(lineStrings);
}


//////////////////////////////////////////////////////////////////
// Calculate the envelope for this geometry
//
MgEnvelope* MgMultiLineString::ComputeEnvelope()
{
    if(m_envelope == NULL)
    {
        m_envelope = MgGeometryUtil::ComputeAggregateEnvelope(this);
    }
    return new MgEnvelope(m_envelope);
}

//////////////////////////////////////////////
// Dispose this object.
//
void MgMultiLineString::Dispose()
{
    delete this;
}

///////////////////////////////////////////////////////////////////////////
// Gets the geometry at the specified position
//
MgGeometry* MgMultiLineString::GetGeometryAt(int index)
{
    return GetLineString(index);
}

MgCoordinateIterator* MgMultiLineString::GetCoordinates()
{
    Ptr<MgCoordinateCollection> coords = new MgCoordinateCollection();
    int count = m_lineStrings->GetCount();

    for (int i = 0; i < count; i++)
    {
        Ptr<MgLineString> lineString = m_lineStrings->GetItem(i);
        Ptr<MgCoordinateIterator> lineIterator = lineString->GetCoordinates();

        while (lineIterator->MoveNext())
        {
            coords->Add(Ptr<MgCoordinate>(lineIterator->GetCurrent()));
        }
    }

    return new MgCoordinateIterator(coords);
}
