//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGMULTILINESTRING_H_
#define _MGMULTILINESTRING_H_

/// \defgroup MgMultiLineString MgMultiLineString
/// \ingroup Geometry_Module_classes
/// \{

///////////////////////////////////////////////////////////////////////////////////////////////
/// \brief
/// An MgMultiLineString is a one-dimensional aggregate geometry
/// whose elements are MgLineString geometries.
///
/// \remarks
/// The line strings
/// are not connected or ordered. An instance of this class is
/// constructed by calling a non-static
/// MgGeometryFactory::CreateMultiLineString() method and, once
/// constructed, is immutable.
///
/// <!-- Example (PHP) -->
/// \htmlinclude PHPExampleTop.html
/// The following code shows the construction of an
/// MgMultiLineString geometry. Review the MgLineString example code.
/// \code
/// // A helper class additional to those created in the
/// // MgLineString example code is needed.
/// $lineStringCollection = new MgLineStringCollection();
///
/// // After each MgLineString geometry is constructed,
/// // it is added to an MgLineStringCollection.
/// $index = $lineStringCollection->Add($lineString);
/// echo "A line string is added to a line string collection at index: $index\n";
///
/// // construct the MgMultiLineString geometry
/// $multiLineString = $geometryFactory->CreateMultiLineString($lineStringCollection);
///
/// // print out the Agf Text string for the geometry
/// $multiLineStringAgfText = $wktReaderWriter->Write($multiLineString);
/// echo "AGF Text representation of MultiLineString: $multiLineStringAgfText\n";
/// \endcode
/// \htmlinclude ExampleBottom.html
///
///
/// <!-- Example (C#) -->
/// \htmlinclude CSharpExampleTop.html
/// The CreateAMultiLineStringXY method calls the CreateALineStringXY method.
/// The code for the CreateALineStringXY method is in the MgLineString example code.
///
/// \code
/// using OSGeo.MapGuide;
///
/// private MgWktReaderWriter wktReaderWriter;
/// private MgGeometryFactory geometryFactory;
/// private MgMultiLineString mlsLs0121Ls1121;
/// // the data for 1 linestring
/// private double[,] da0121 = { { 0, 1 }, { 2, 1 } };
/// // the data for another linestring
/// private double[,] da1121 = { { 1, 1 }, { 2, 1 } };
/// // the data for 2 linestrings
/// private double[][,] mlsLs0121Ls1121Data;
/// private String geometryAgfText;
///
/// public MgMultiLineString CreateAMultiLineStringXY(double[][,] lineStrings)
/// {
///     MgLineStringCollection lines = new MgLineStringCollection();
///     for (int i = 0; i < lineStrings.GetLength(0); i++)
///     {
///         lines.Add(CreateALineStringXY(lineStrings[i]));
///     }
///     return geometryFactory.CreateMultiLineString(lines);
/// }
///
/// geometryFactory = new MgGeometryFactory();
/// mlsLs0121Ls1121Data = new double[2][,];
/// mlsLs0121Ls1121Data[0] = da0121;
/// mlsLs0121Ls1121Data[1] = da1121;
/// mlsLs0121Ls1121 = CreateAMultiLineStringXY(mlsLs0121Ls1121Data);
///
/// // print out the Agf Text string for the geometry
/// wktReaderWriter = new MgWktReaderWriter();
/// geometryAgfText = wktReaderWriter.Write(mlsLs0121Ls1121);
/// // geometryAgfText now contains:
/// // "MULTILINESTRING XY ((0 1, 2 1), (1 1, 2 1))"
/// \endcode
/// \htmlinclude ExampleBottom.html
///
class MG_GEOMETRY_API MgMultiLineString : public MgAggregateGeometry
{
public:

    virtual int GetCount();  /// __get, __inherited

    //////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the line string at the specified index. The indexing
    /// starts at zero.
    ///
    /// \param index (int)
    /// An integer specifying the location of the line
    /// string in the collection.
    ///
    /// \return
    /// The MgLineString at the specified index.
    ///
    virtual MgLineString* GetLineString(int index);


    virtual int GetGeometryType();  /// __get, __inherited

    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    virtual MgGeometricEntity* Transform(MgTransform* transform);

public:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgMultiLineString object
    ///
    MgMultiLineString(MgLineStringCollection* lineStrings);

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgMultiLineString object for deserialization
    ///
    MgMultiLineString();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the geometry at the specified position
    ///
    virtual MgGeometry* GetGeometryAt(int index);

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator over the coordinates included in this
    /// geometric entity.
    ///
    /// \return
    /// The coordinates of this geometric entity
    ///
    virtual MgCoordinateIterator* GetCoordinates();

protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    /// \return
    /// Class Identifider.
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgLineStringCollection>   m_lineStrings;
};
/// \}

#endif //_MGMULTILINESTRING_H_
