//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGMULTIPOINT_H_
#define _MGMULTIPOINT_H_

/// \defgroup MgMultiPoint MgMultiPoint
/// \ingroup Geometry_Module_classes
/// \{

///////////////////////////////////////////////////////////////////
/// \brief
/// An MgMultiPoint is a 0 dimensional aggregate geometry whose
/// elements are MgPoint geometries.
///
/// \remarks
/// The points are not connected
/// or ordered. An instance of this class is constructed by
/// calling a non-static MgGeometryFactory::CreateMultiPoint()
/// method and, once constructed, is immutable.
///
/// <!-- Example (PHP) -->
/// \htmlinclude PHPExampleTop.html
/// The following code shows the construction of an MgMultiPoint
/// geometry. Review the MgPoint Class example code.
/// \code
/// // A helper class additional to those created in the
/// // MgPoint example code is needed.
/// $pointCollection = new MgPointCollection();
///
/// // After each MgPoint geometry is constructed,
/// // it is added to an MgPointCollection.
/// $index = $pointCollection->Add($point);
/// echo "A point is added to a point collection at index: $index\n";
///
/// // construct the MgMultiPoint geometry
/// $multiPoint = $geometryFactory->CreateMultiPoint($pointCollection);
///
/// // print out the Agf Text string for the geometry
/// $multiPointAgfText = $wktReaderWriter->Write($multiPoint);
///
/// echo "AGF Text representation of MultiPoint: $multiPointAgfText\n";
/// \endcode
/// \htmlinclude ExampleBottom.html
///
/// <!-- Example (C#) -->
/// \htmlinclude CSharpExampleTop.html
/// The CreateAMultiPointXY method calls the CreateAPointXY method.
/// The code for the CreateAPointXY method is in the MgPoint example code.
///
/// \code
/// using OSGeo.MapGuide;
///
/// private MgWktReaderWriter wktReaderWriter;
/// private MgGeometryFactory geometryFactory;
/// private MgMultiPoint mpt1121;
/// private double[,] da1121 = { { 1, 1 }, { 2, 1 } };
/// private String geometryAgfText;
///
/// public MgMultiPoint CreateAMultiPointXY(double[,] multiPointData)
/// {
///     MgPointCollection points = new MgPointCollection();
///     for (int i = 0; i < multiPointData.GetLength(0); i++)
///     {
///         points.Add(CreateAPointXY(multiPointData[i, 0], multiPointData[i, 1]));
///     }
///     return geometryFactory.CreateMultiPoint(points);
/// }
///
/// geometryFactory = new MgGeometryFactory();
/// mpt1121 = CreateAMultiPointXY(da1121);
///
/// // print out the Agf Text string for the geometry
/// wktReaderWriter = new MgWktReaderWriter();
/// geometryAgfText = wktReaderWriter.Write(mpt1121);
/// // geometryAgfText now contains:
/// // "MULTIPOINT XY ( 1 1, 2 1 )"
/// \endcode
/// \htmlinclude ExampleBottom.html
///
class MG_GEOMETRY_API MgMultiPoint : public MgAggregateGeometry
{
public:

    virtual int GetCount();  /// __get, __inherited

    ///////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the point at the specified index. The indexing starts at
    /// zero.
    ///
    /// \param index (int)
    /// An integer specifying the location of the point
    /// in the collection.
    ///
    /// \return
    /// The MgPoint at the specified index.
    ///
    virtual MgPoint* GetPoint(int index);

    virtual int GetGeometryType();  /// __get, __inherited

    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this geometric entity.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this geometric entity.
    ///
    /// \return
    /// An MgGeometricEntity that is a copy of this one with each coordinate
    /// transformed.
    ///
    virtual MgGeometricEntity* Transform(MgTransform* transform);

public:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgMultiPoint object
    ///
    MgMultiPoint(MgPointCollection* points);

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgMultiPoint object for deserialization
    ///
    MgMultiPoint();


    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the geometry at the specified position
    ///
    virtual MgGeometry* GetGeometryAt(int index);

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator over the coordinates included in this
    /// geometric entity.
    ///
    /// \return
    /// The coordinates of this geometric entity
    ///
    virtual MgCoordinateIterator* GetCoordinates();

protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    /// \return
    /// MgEnvelope.
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgPointCollection>   m_points;
};
/// \}

#endif //_MGMULTIPOINT_H_
