//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGPOINT_H_
#define _MGPOINT_H_

/// \defgroup MgPoint MgPoint
/// \ingroup Geometry_Module_classes
/// \{

//////////////////////////////////////////////////////////////
/// \brief
/// An MgPoint is a 0-dimensional geometry and represents a
/// single location in coordinate space.
///
/// \remarks
/// An instance of this
/// class is constructed by calling a non-static
/// MgGeometryFactory::CreatePoint() method and, once
/// constructed, is immutable.
///
/// <!-- Example (PHP) -->
/// \htmlinclude PHPExampleTop.html
/// The following code shows the construction of a point
/// \code
/// $geometryFactory = new MgGeometryFactory();
///
/// // create a coordinate
/// $oordinate = $geometryFactory->CreateCoordinateXY(0,2);
///
/// // create a point
/// $point = $geometryFactory->CreatePoint($coordinate);
///
/// // print out the Agf Text string for the geometry
/// $pointAgfText = $wktReaderWriter->Write($point);
/// echo "AGF Text representation of Point: $pointAgfText\n";
/// \endcode
/// \htmlinclude ExampleBottom.html
///
/// <!-- Example (C#) -->
/// \htmlinclude CSharpExampleTop.html
/// \code
/// using OSGeo.MapGuide;
///
/// private MgPoint pt11;
/// private MgPoint pt11FromText;
/// private String pt11TextSpec = "POINT XY ( 1 1 )";
/// private MgWktReaderWriter wktReaderWriter;
/// private MgGeometryFactory geometryFactory;
/// private String geometryAgfText;
///
///
/// public MgPoint CreateAPointXY(double x, double y)
/// {
///     MgCoordinate coord = geometryFactory.CreateCoordinateXY(x, y);
///     return geometryFactory.CreatePoint(coord);
/// }
///
/// wktReaderWriter = new MgWktReaderWriter();
/// geometryFactory = new MgGeometryFactory();
/// // create a geometry using the geometry factory
/// pt11 = CreateAPointXY(1, 1);
/// // create a geometry from an Agf textual specification
/// pt11FromText = wktReaderWriter.Read(pt11TextSpec) as MgPoint;
/// // print out the Agf Text string for the geometry
/// geometryAgfText = wktReaderWriter.Write(pt11);
/// // geometryAgfText now contains:
/// // "POINT XY ( 1 1 )"
/// \endcode
/// \htmlinclude ExampleBottom.html
///
class MG_GEOMETRY_API MgPoint : public MgGeometry
{
public:

    virtual int GetGeometryType();  /// __get, __inherited

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the coordinate that defines this point.
    ///
    /// \return
    /// The MgCoordinate that defines this point.
    ///
    virtual MgCoordinate* GetCoordinate();  /// __get

    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this geometric entity.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this geometric entity.
    ///
    /// \return
    /// An MgGeometricEntity that is a copy of this one with each coordinate
    /// transformed.
    ///
    virtual MgGeometricEntity* Transform(MgTransform* transform);

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator over the coordinates included in this
    /// geometric entity.
    ///
    /// \return
    /// The coordinates of this geometric entity
    ///
    virtual MgCoordinateIterator* GetCoordinates();  /// __get

public:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgPoint object
    ///
    MgPoint(MgCoordinate* coordinate);

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgPoint object for deserialization
    ///
    MgPoint();

protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    /// \return
    /// Class Identifider.
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgCoordinate> m_coord;
};
/// \}

#endif //_MGPOINT_H_
