//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_POINTCOLLECTION_H_
#define _MG_POINTCOLLECTION_H_

/// \defgroup MgPointCollection MgPointCollection
/// \ingroup Collections_Module
/// \{

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgPointCollection provides support for defining an ordered set of
/// points.
class MG_GEOMETRY_API MgPointCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of points in the collection.
    ///
    /// \return
    /// Returns number of points in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the point in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the point to retrieve.
    ///
    /// \return
    /// Returns the point in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgPoint* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the point in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to set.
    /// \param value (MgPoint)
    /// point to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgPoint* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified coordinate to the end of the collection.
    ///
    /// \param value (MgPoint)
    /// point to add to the collection.
    ///
    /// \return
    /// Returns nothing.
    ///
    virtual void Add(MgPoint* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified point at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the coordinate.
    /// \param value (MgPoint)
    /// point to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgPoint* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all points from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified point from the collection.
    ///
    /// \param value (MgPoint)
    /// point reference to remove.
    ///
    /// \return
    /// Returns true if removal was successful.
    ///
    virtual bool Remove(const MgPoint* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the point at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the point to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified point, false
    /// otherwise.
    ///
    /// \param value (MgPoint)
    /// point reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified point, false
    /// otherwise.
    ///
    virtual bool Contains(const MgPoint* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified point in the collection or -1
    /// if the point does not exist.
    ///
    /// \param value (MgPoint)
    /// point reference to find the index of.
    ///
    /// \return
    /// The index of the specified point in the collection or -1 if the
    /// point does not exist.
    ///
    virtual int IndexOf(const MgPoint* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgPointCollection object
    ///
    MgPointCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgPointCollection destructor
    ///
    virtual ~MgPointCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgPointCollection* Copy();

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    void Dispose();

private:

    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif // _MG_POINTCOLLECTION_H_
