//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#include "stdafx.h"

//////////////////////////////////////////////////////////////////
// Construct a MgPolygon object
//
MgPolygon::MgPolygon(MgLinearRing* outerRing, MgLinearRingCollection* innerRings)
{
    m_outerRing = (MgLinearRing*)outerRing->Copy();
    m_innerRings = innerRings != NULL? innerRings->Copy(): NULL;
}

//////////////////////////////////////////////////////////////////
// Construct an empty MgPolygon object for deserialization
//
MgPolygon::MgPolygon()
{
}

///////////////////////////////////////////////////////////////////////////
// Gets the exterior ring of the polygon.
//
MgLinearRing* MgPolygon::GetExteriorRing()
{
    return SAFE_ADDREF((MgLinearRing*)m_outerRing);
}

///////////////////////////////////////////////////////////////////////////
// Gets the number of interior rings in this polygon.
//
int MgPolygon::GetInteriorRingCount()
{
    return (m_innerRings != NULL)? m_innerRings->GetCount(): 0;
}

///////////////////////////////////////////////////////////////////////////
// Gets the interior ring at the specified index. The indexing starts at
// zero.
//
MgLinearRing* MgPolygon::GetInteriorRing(int index)
{
    if (m_innerRings == NULL)
        throw new MgException(L"ArgumentOutOfRange");

    return m_innerRings->GetItem(index);
}

///////////////////////////////////////////////////////////////////////////
// Gets the type of this geometry.
//
int MgPolygon::GetGeometryType()
{
    return MgGeometryType::Polygon;
}

///////////////////////////////////////////////////////////////////////////
// Tests whether this entity is empty or not.
//
bool MgPolygon::IsEmpty()
{
    return false;
}

///////////////////////////////////////////////////////////////////////////
// Tests whether the entities boundary encloses an area.
//
bool MgPolygon::IsClosed()
{
    return true;
}

///////////////////////////////////////////////////////////////////////////
// Returns a copy of this geometric entity.
//
MgGeometricEntity* MgPolygon::Copy()
{
    return new MgPolygon(m_outerRing, m_innerRings);
}

///////////////////////////////////////////////////////////////////////////
// Returns a transformed copy of this geometric entity.
//
MgGeometricEntity* MgPolygon::Transform(MgTransform* transform)
{
    Ptr<MgLinearRing> newOuterRing = (MgLinearRing*)m_outerRing->Transform(transform);
    Ptr<MgLinearRingCollection> newInnerRings;

    if (m_innerRings != NULL)
    {
        newInnerRings = new MgLinearRingCollection();
        int count = m_innerRings->GetCount();
        for (int i=0; i<count; ++i)
        {
            Ptr<MgLinearRing> innerRing = m_innerRings->GetItem(i);
            Ptr<MgLinearRing> newInnerRing = (MgLinearRing*)innerRing->Transform(transform);
            newInnerRings->Add(newInnerRing);
        }
    }

    return new MgPolygon(newOuterRing, newInnerRings);
}


//////////////////////////////////////////////////////////////////
// Calculate the envelope for this geometry
//
MgEnvelope* MgPolygon::ComputeEnvelope()
{
    if (m_envelope == NULL)
        m_envelope = m_outerRing->Envelope();

    return new MgEnvelope(m_envelope);
}

//////////////////////////////////////////////
// Dispose this object.
//
void MgPolygon::Dispose()
{
    delete this;
}


//////////////////////////////////////////////////////////////////
///<summary>
/// Convert to AWKT representation


//////////////////////////////////////////////////////////////////
MgCoordinateIterator* MgPolygon::GetCoordinates()
{
    Ptr<MgCoordinateCollection> coords = new MgCoordinateCollection();
    Ptr<MgCoordinateIterator> outerCoords = m_outerRing->GetCoordinates();

    while (outerCoords->MoveNext())
    {
        coords->Add(Ptr<MgCoordinate>(outerCoords->GetCurrent()));
    }

    if (m_innerRings != NULL)
    {
        int innerCount = m_innerRings->GetCount();
        for (int i=0; i<innerCount; ++i)
        {
            Ptr<MgLinearRing> ring = m_innerRings->GetItem(i);
            Ptr<MgCoordinateIterator> innerCoords = ring->GetCoordinates();

            while (innerCoords->MoveNext())
            {
                coords->Add(Ptr<MgCoordinate>(innerCoords->GetCurrent()));
            }
        }
    }

    return new MgCoordinateIterator(coords);
}
