//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MGPOLYGON_H_
#define _MGPOLYGON_H_

/// \defgroup MgPolygon MgPolygon
/// \ingroup Geometry_Module_classes
/// \{

/////////////////////////////////////////////////////////////////
/// \brief
/// An MgPolygon is a region, defined by 1 exterior boundary and
/// 0 or more interior boundaries.
///
/// \remarks
/// Each interior boundary defines
/// a hole in the polygon. The interior and exterior boundaries
/// are defined with linear interpolation between points. Each
/// consecutive pair of points defines a line segment. An object
/// of this type is constructed by calling a non-static
/// MgGeometryFactory::CreatePolygon() method and, once
/// constructed, is immutable.
///
/// <!-- Example (PHP) -->
/// \htmlinclude PHPExampleTop.html
/// The following example code creates a square polygon, whose
/// side is 3 units in length, and which contains a "hole" at its
/// center. The "hole" is also a square, whose side is 1 unit in
/// length.
///
/// \code
/// $geometryFactory = new MgGeometryFactory();
/// $coordinateCollection = new MgCoordinateCollection();
/// $interiorRingCollection = new MgLinearRingCollection();
/// $wktReaderWriter = new MgWktReaderWriter();
/// // note that the coordinates for the external boundary
/// // are added to the coordinate collection in a
/// // counterclockwise order
/// // create first coordinate for external boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(0,0);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create second coordinate for external boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(3,0);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create third coordinate for external boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(3,3);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create fourth coordinate for external boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(0,3);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create fifth coordinate for external boundary that is the same as
/// // the first to close the boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(0,0);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create LinearRing that defines the external boundary
/// $exteriorRing = $geometryFactory->CreateLinearRing($coordinateCollection);
/// // clear the collection before reusing for internal boundary
/// $coordinateCollection->Clear();
///
/// // note that the coordinates for the internal boundary
/// // are added to the coordinate collection in a
/// // clockwise order
/// // create first coordinate for internal boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(1,1);
///
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create second coordinate for internal boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(1,2);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create third coordinate for internal boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(2,2);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create fourth coordinate for internal boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(2,1);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create fifth coordinate for internal boundary that is the same as
/// // the first to close the boundary
/// $coordinate = $geometryFactory->CreateCoordinateXY(1,1);
/// $index = $coordinateCollection->Add($coordinate);
/// echo "Coordinate added to coordinate collection at index: $index\n";
///
/// // create LinearRing that defines the internal boundary
/// $interiorRing =
/// $geometryFactory->CreateLinearRing($coordinateCollection);
///
/// // add the interior ring to the interior ring collection
/// $index = $interiorRingCollection->Add($interiorRing);
///
/// // create the MgPolygon geometry
/// $polygon = $geometryFactory->CreatePolygon($exteriorRing, $interiorRingCollection);
///
/// // print out the Agf Text string for the geometry
/// $polygonAgfText = $wktReaderWriter->Write($polygon);
/// echo "AGF Text representation of line string: $polygonAgfText\n";
/// \endcode
/// \htmlinclude ExampleBottom.html
///
/// <!-- Example (C#) -->
/// \htmlinclude CSharpExampleTop.html
/// The following example code creates a square polygon, whose
/// side is 3 units in length, and which contains a "hole" at its
/// center. The "hole" is also a square, whose side is 1 unit in
/// length.
/// \code
/// using OSGeo.MapGuide;
///
/// private MgWktReaderWriter wktReaderWriter;
/// private MgGeometryFactory geometryFactory;
/// // this polygon has one exterior ring and one interior ring
/// private MgPolygon pner1141441411ir2223333222;
/// // the following array is used for the exterior ring of a polygon
/// // the points are in counter-clockwise order
/// private double[,] da1141441411 = { { 1, 1 }, { 4, 1 }, { 4, 4 }, { 1, 4 }, { 1, 1 } };
/// // the following array is used for the interior ring of a polygon
/// // the points are in clockwise order
/// private double[,] da2223333222 = { { 2, 2 }, { 2, 3 }, { 3, 3 }, { 3, 2 }, { 2, 2 } };
/// // this array contains an array that will be used for an exterior ring
/// // and one that will be used for an interior ring
/// private double[][,] pnEr1141441411Ir2223333222Data;
/// private String geometryAgfText;
///
/// public MgLinearRing CreateALinearRingXY(double[,] ringData)
/// {
///     MgCoordinateCollection coords = new MgCoordinateCollection();
///     for (int i = 0; i < ringData.GetLength(0); i++)
///     {
///         coords.Add(geometryFactory.CreateCoordinateXY(ringData[i, 0], ringData[i, 1]));
///     }
///     return geometryFactory.CreateLinearRing(coords);
/// }
///
/// public MgPolygon CreateAPolygonXY(double[][,] ringsData)
/// {
///     MgLinearRing exteriorRing = CreateALinearRingXY(ringsData[0]);
///     MgLinearRingCollection interiorRings = new MgLinearRingCollection();
///     for (int i = 1; i < ringsData.GetLength(0); i++)
///     {
///         interiorRings.Add(CreateALinearRingXY(ringsData[i]));
///     }
///     return geometryFactory.CreatePolygon(exteriorRing, interiorRings);
/// }
///
/// geometryFactory = new MgGeometryFactory();
/// pnEr1141441411Ir2223333222Data = new double[2][,];
/// pnEr1141441411Ir2223333222Data[0] = da1141441411;
/// pnEr1141441411Ir2223333222Data[1] = da2223333222;
/// // create a polygon using the geometry factory
/// pner1141441411ir2223333222 = CreateAPolygonXY(pnEr1141441411Ir2223333222Data);
///
/// // print out the Agf Text string for the geometry
/// wktReaderWriter = new MgWktReaderWriter();
/// geometryAgfText = wktReaderWriter.Write(pner1141441411ir2223333222);
/// // geometryAgfText now contains:
/// // "POLYGON XY ((1 1, 4 1, 4 4, 1 4, 1 1), (2 2, 2 3, 3 3, 3 2, 2 2))"
/// \endcode
/// \htmlinclude ExampleBottom.html
///
class MG_GEOMETRY_API MgPolygon : public MgGeometry
{
public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the exterior ring of the polygon.
    ///
    /// \return
    /// An MgLinearRing that defines the exterior ring of the polygon.
    ///
    virtual MgLinearRing* GetExteriorRing();  /// __get

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of interior rings in this polygon.
    ///
    /// \return
    /// The number of interior rings.
    ///
    virtual int GetInteriorRingCount();  /// __get

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the interior ring at the specified index. The indexing
    /// starts at zero.
    ///
    /// \param index (int)
    /// An integer specifying the location of the
    /// interior ring in the collection.
    ///
    /// \return
    /// The MgLinearRing at the specified index.
    ///
    virtual MgLinearRing* GetInteriorRing(int index);

    virtual int GetGeometryType();  /// __get, __inherited

    virtual bool IsEmpty();

    virtual bool IsClosed();

    virtual MgGeometricEntity* Copy();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a transformed copy of this geometric entity.
    ///
    /// \param transform (MgTransform)
    /// The MgTransform to use in constructing a copy of this geometric entity.
    ///
    /// \return
    /// An MgGeometricEntity that is a copy of this one with each coordinate
    /// transformed.
    ///
    virtual MgGeometricEntity* Transform(MgTransform* transform);

    /////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns an iterator over the coordinates included in this
    /// geometric entity.
    ///
    /// \return
    /// The coordinates of this geometric entity
    ///
    virtual MgCoordinateIterator* GetCoordinates();

public:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgPolygon object
    ///
    MgPolygon(MgLinearRing* outerRing, MgLinearRingCollection* innerRings);

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct an empty MgPolygon object for deserialization
    ///
    MgPolygon();

protected:

    //////////////////////////////////////////////////////////////////
    /// \brief
    /// Calculate the envelope for this geometry
    ///
    /// \return
    /// Class Identifider.
    ///
    virtual MgEnvelope* ComputeEnvelope();

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    virtual void Dispose();

private:
    Ptr<MgLinearRing> m_outerRing;
    Ptr<MgLinearRingCollection> m_innerRings;
};
/// \}

#endif //_MGPOLYGON_H_
