//
//  Copyright (C) 2004-2011 by Autodesk, Inc.
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of version 2.1 of the GNU Lesser
//  General Public License as published by the Free Software Foundation.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//

#ifndef _MG_POLYGONCOLLECTION_H_
#define _MG_POLYGONCOLLECTION_H_

/// \defgroup MgPolygonCollection MgPolygonCollection
/// \ingroup Collections_Module
/// \{

///////////////////////////////////////////////////////////////////////////////
/// \brief
/// MgPolygonCollection provides support for defining an ordered set of
/// polygons.
///
class MG_GEOMETRY_API MgPolygonCollection : public MgDisposable
{

public:
    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the number of polygons in the collection.
    ///
    /// \return
    /// Returns number of polygons in the collection.
    ///
    virtual int GetCount() const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Gets the polygon in the collection at the specified index. Throws
    /// an invalid argument exception if the index is out of range.
    ///
    /// \param index (int)
    /// Index of the polygon to retrieve.
    ///
    /// \return
    /// Returns the polygon in the collection at the specified index.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual MgPolygon* GetItem(int index) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Sets the polygon in the collection at the specified index to the
    /// specified value. Throws an invalid argument exception if the index is
    /// out of range.
    ///
    /// \param index (int)
    /// Index of the coordinate to set.
    /// \param value (MgPolygon)
    /// polygon to place into the collection.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void SetItem(int index, MgPolygon* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Adds the specified coordinate to the end of the collection.
    ///
    /// \param value (MgPolygon)
    /// polygon to add to the collection.
    ///
    /// \return
    /// Returns nothing
    ///
    virtual void Add(MgPolygon* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Inserts the specified polygon at the specified index within the
    /// collection. Items following the insertion point are moved down to
    /// accommodate the new item. Throws an invalid argument exception if the
    /// specified index is out of range.
    ///
    /// \param index (int)
    /// Index at which to insert the coordinate.
    /// \param value (MgPolygon)
    /// polygon to insert.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void Insert(int index, MgPolygon* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes all polygons from the collection.
    ///
    virtual void Clear();

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the specified polygon from the collection.
    ///
    /// \param value (MgPolygon)
    /// polygon reference to remove.
    ///
    /// \return
    /// Returns true if removal was successful.
    ///
    virtual bool Remove(const MgPolygon* value);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Removes the polygon at the specified index from the collection.
    /// Throws an invalid argument exception if the index does not exist within
    /// the collection.
    ///
    /// \param index (int)
    /// Index of the polygon to remove.
    ///
    /// \exception MgIndexOutOfRangeException
    ///
    virtual void RemoveAt(int index);

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns true if the collection contains the specified polygon, false
    /// otherwise.
    ///
    /// \param value (MgPolygon)
    /// polygon reference to check containment of.
    ///
    /// \return
    /// True if the collection contains the specified polygon, false
    /// otherwise.
    ///
    virtual bool Contains(const MgPolygon* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns the index of the specified polygon in the collection or -1
    /// if the polygon does not exist.
    ///
    /// \param value (MgPolygon)
    /// polygon reference to find the index of.
    ///
    /// \return
    /// The index of the specified polygon in the collection or -1 if the
    /// polygon does not exist.
    ///
    virtual int IndexOf(const MgPolygon* value) const;

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Construct a MgPolygonCollection object
    ///
    MgPolygonCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// MgPolygonCollection destructor
    ///
    virtual ~MgPolygonCollection();

public:

    ///////////////////////////////////////////////////////////////////////////
    /// \brief
    /// Returns a copy of this collection
    ///
    MgPolygonCollection* Copy();

protected:

    //////////////////////////////////////////////
    /// \brief
    /// Dispose this object.
    ///
    void Dispose();

private:

    Ptr<MgDisposableCollection> m_collection;
};
/// \}

#endif // _MG_POLYGONCOLLECTION_H_
